﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace MyLibrary
{
    public partial class MainForm : Form
    {
        private bool unsavedChanges;
        private string file;

        public MainForm()
        {
            InitializeComponent();

            unsavedChanges = false;
        }

        protected override void OnClosing(CancelEventArgs e)
        {
            base.OnClosing(e);

            if (unsavedChanges)
            {
                DialogResult result = MessageBox.Show("Save changes before closing?", "Unsaved Data", MessageBoxButtons.YesNoCancel, MessageBoxIcon.Warning);

                switch (result)
                {
                    case DialogResult.Yes:
                        {
                            saveToolStripMenuItem.PerformClick();

                            break;
                        }
                    case DialogResult.No:
                        {
                            break;
                        }
                    case DialogResult.Cancel:
                        {
                            e.Cancel = true;
                            break;
                        }
                    default: break;
                }
            }
        }

        private void bookListBox_SelectedValueChanged(object sender, EventArgs e)
        {
            Book selectedBook = bookListBox.SelectedItem as Book;

            if (selectedBook != null)
            {
                titleLabel.Text = selectedBook.Title;
                yearLabel.Text = String.Format("({0})", selectedBook.PublicationDate.Year);
                authorLabel.Text = String.Format("by {0}", ((selectedBook.Author == null) ? "unknown author" : selectedBook.Author));
                isbnLabel.Text = String.Format("ISBN: {0}", selectedBook.ISBN);

                titleLabel.Visible = true;
                yearLabel.Visible = true;
                authorLabel.Visible = true;
                isbnLabel.Visible = true;

                removeToolStripMenuItem.Enabled = true;
            }
            else
            {
                titleLabel.Visible = false;
                yearLabel.Visible = false;
                authorLabel.Visible = false;
                isbnLabel.Visible = false;

                removeToolStripMenuItem.Enabled = false;
            }
        }

        private void newToolStripMenuItem_Click(object sender, EventArgs e)
        {
            NewBookDialog diag = new NewBookDialog();

            if (diag.ShowDialog() == DialogResult.OK)
            {
                bookListBox.Items.Add(diag.Book);

                unsavedChanges = true;
            }
        }

        private void removeToolStripMenuItem_Click(object sender, EventArgs e)
        {
            bookListBox.Items.Remove(bookListBox.SelectedItem);

            unsavedChanges = true;
        }

        private void LoadData(string fileName)
        {
            try
            {
                using (StreamReader sr = new StreamReader(fileName))
                {
                    string line;
                    while ((line = sr.ReadLine()) != null)
                    {
                        string[] data = line.Split(';');

                        Book book = new Book()
                            {
                                Title = data[0],
                                PublicationDate = DateTime.Parse(data[1]),
                                ISBN = data[2],
                                Author = ((data.Length > 3) ? data[3] : null)
                            };

                        bookListBox.Items.Add(book);
                    }
                }
            }
            catch (IOException e)
            {
                MessageBox.Show("Error", e.Message);
            }
        }

        private void SaveData(string fileName)
        {
            try
            {
                using (StreamWriter sw = new StreamWriter(fileName))
                {
                    foreach (object item in bookListBox.Items)
                    {
                        Book book = item as Book;

                        if (book != null)
                        {
                            sw.Write("{0};{1};{2}", book.Title, book.PublicationDate, book.ISBN);
                            if (book.Author != null)
                            {
                                sw.WriteLine(";{0}", book.Author);
                            }
                            else
                            {
                                sw.WriteLine();
                            }
                        }
                    }
                }
            }
            catch (IOException e)
            {
                MessageBox.Show("Error", e.Message);
            }
        }

        private void quitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Close();
        }

        private void SaveAs()
        {
            SaveFileDialog diag = new SaveFileDialog();

            diag.AddExtension = true;
            diag.DefaultExt = ".bdb";
            diag.Title = "Save as";
            diag.Filter = "Book database (*.bdb)|*.bdb";

            if (diag.ShowDialog() == DialogResult.OK)
            {
                SaveData(diag.FileName);

                unsavedChanges = false;

                file = diag.FileName;
            }
        }

        private void saveToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (file != null)
            {
                SaveData(file);
            }
            else
            {
                SaveAs();
            }
        }

        private void loadToolStripMenuItem_Click(object sender, EventArgs e)
        {
            OpenFileDialog diag = new OpenFileDialog();

            diag.Title = "Select Database File";
            diag.Filter = "Book database (*.bdb)|*.bdb";

            if (diag.ShowDialog() == DialogResult.OK)
            {
                bookListBox.Items.Clear();

                LoadData(diag.FileName);

                unsavedChanges = false;

                file = diag.FileName;
            }
        }

        private void saveAsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            SaveAs();
        }
    }
}
