﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace MyLibrary
{
    public partial class NewBookDialog : Form
    {
        public Book Book { get; private set; }

        public NewBookDialog()
        {
            InitializeComponent();
        }

        private void titleTextBox_Validating(object sender, CancelEventArgs e)
        {
            if (String.IsNullOrWhiteSpace(titleTextBox.Text))
            {
                errorProvider.SetError(titleTextBox, "Title cannot be blank.");
                e.Cancel = true;
            }
            else
            {
                errorProvider.SetError(titleTextBox, null);
            }
        }

        private void publicationDatePicker_Validating(object sender, CancelEventArgs e)
        {
            if (publicationDatePicker.Value > DateTime.Now)
            {
                errorProvider.SetError(publicationDatePicker, "Book couldn't have been published in future!");
                e.Cancel = true;
            }
            else
            {
                errorProvider.SetError(publicationDatePicker, null);
            }
        }

        private void isbnTextBox_Validating(object sender, CancelEventArgs e)
        {
            string isbn = isbnTextBox.Text;

            if (!isbn.All(ch => Char.IsDigit(ch)))
            {
                errorProvider.SetError(isbnTextBox, "Invalid ISBN. Only decimal digits are allowed!");
                e.Cancel = true;
            }
            else if ((isbn.Length != 10) && (isbn.Length != 13))
            {
                errorProvider.SetError(isbnTextBox, "Invalid ISBN. Must be either 10 or 13 digits long.");
                e.Cancel = true;
            }
            else
            {
                errorProvider.SetError(isbnTextBox, null);
            }
        }

        private void cancelButton_Click(object sender, EventArgs e)
        {
            DialogResult = DialogResult.Cancel;

            Close();
        }

        private void okButton_Click(object sender, EventArgs e)
        {
            if (!ValidateChildren())
            {
                return;
            }

            Book = new Book()
                {
                    Title = titleTextBox.Text,
                    Author = (String.IsNullOrWhiteSpace(authorTextBox.Text) ? null : authorTextBox.Text),
                    PublicationDate = publicationDatePicker.Value,
                    ISBN = isbnTextBox.Text
                };

            DialogResult = DialogResult.OK;

            Close();
        }
    }
}
